/*
 * Copyright (C) 2022 The Libphonenumber Authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.i18n.phonenumbers.internal;

import junit.framework.TestCase;

public class GeoEntityUtilityTest extends TestCase {

  public void test_isGeoEntity_shouldReturnTrueForCountryRegionCode() {
    assertTrue(GeoEntityUtility.isGeoEntity("DE"));
  }

  public void test_isGeoEntity_shouldReturnFalseForWorldRegionCode() {
    assertFalse(GeoEntityUtility.isGeoEntity("001"));
  }

  public void test_isGeoEntity_shouldReturnTrueForCountryCallingCode() {
    assertTrue(GeoEntityUtility.isGeoEntity(41));
  }

  public void test_isGeoEntity_shouldReturnFalseForInternationalSharedCostServiceCallingCode() {
    assertFalse(GeoEntityUtility.isGeoEntity(808));
  }

  public void test_isGeoEntity_shouldReturnFalseForNonExistingCountryCallingCode() {
    assertFalse(GeoEntityUtility.isGeoEntity(111111111));
  }
}
